import java.util.Set;

public class ExampleTM {
	
	private static final Set<Character> ALPHABET = Set.of('X');

	private static final TuringMachine.Direction R = TuringMachine.Direction.R;
	private static final TuringMachine.Direction L = TuringMachine.Direction.L;
	private static final TuringMachine.Direction N = TuringMachine.Direction.N;

	public static TuringMachine getMachine() {
		/*
		 * This TM computes the following function on words from {X}*:
		 * If the length of the input word is even, the word is returned as it is.
		 * If the length of the input word is odd, one X is removed and the result returned. 
		 */
		
		TuringMachine.State even = new TuringMachine.State("even");
		TuringMachine.State odd = new TuringMachine.State("odd");
		TuringMachine.State goLeft = new TuringMachine.State("←");
		TuringMachine.State delete = new TuringMachine.State("del");
		TuringMachine.State halt = new TuringMachine.State("HALT");
		
		TuringMachine tm = new TuringMachine(ALPHABET, even);
		tm.addFinalState(halt);

		// go through word left-to-right, switching between even and odd state
		tm.addTransition(even, 'X', odd, 'X', R);
		tm.addTransition(odd, 'X', even, 'X', R);
		
		// if end of word is reached: either go left directly or delete rightmost X and then go left
		tm.addTransition(even, ' ', goLeft, ' ', L);
		tm.addTransition(odd, ' ', delete, ' ', L);
		
		// delete one X
		tm.addTransition(delete, 'X', goLeft, ' ', L);
		
		// go all the way to the left to the first X
		tm.addTransition(goLeft, 'X', goLeft, 'X', L);
		tm.addTransition(goLeft, ' ', halt, ' ', R);
		
		return tm;
	}
	
	public static TuringMachineRunner.Tape run(String w) {
		try {
			return TuringMachineRunner.run(getMachine(), Integer.MAX_VALUE, new TuringMachineRunner.Tape(w));
		} catch (TuringMachineRunner.RunException e) {
			System.err.println(e.getMessage());
			return null;
		}
	}
	
	public static void test(String w) {
		System.out.println(run(w));
	}
	
	public static void test() {
		test("");
		test("X");
		test("XX");
		test("XXX");
		test("XXXX");
		test("XXXXX");
	}
	
	public static void main(String[] args) {
		test();
	}
	
}
